@@ -15,6 +15,8 @@ from django.db.models.query import QuerySet
 
 from django.test import TestCase
 from django.db.models import Q, Count
+if django.VERSION >= (1, 8):
+    from django.db.models import Case, When
 from django.db import models
 from django.contrib.contenttypes.models import ContentType
 from django.utils import six
@@ -186,6 +188,18 @@ class ModelWithMyManager(ShowFieldTypeAndContent, Model2A):
     objects = MyManager()
     field4 = models.CharField(max_length=10)
 
+
+class ModelWithMyManagerNoDefault(ShowFieldTypeAndContent, Model2A):
+    objects = PolymorphicManager()
+    my_objects = MyManager()
+    field4 = models.CharField(max_length=10)
+
+class ModelWithMyManagerDefault(ShowFieldTypeAndContent, Model2A):
+    my_objects = MyManager()
+    objects = PolymorphicManager()
+    field4 = models.CharField(max_length=10)
+
+
 if django.VERSION >= (1, 7):
     class ModelWithMyManager2(ShowFieldTypeAndContent, Model2A):
         objects = MyManagerQuerySet.as_manager()
@@ -406,6 +420,11 @@ class CustomPkInherit(CustomPkBase):
     i = models.CharField(max_length=1)
 
 
+class DateModel(PolymorphicModel):
+
+    date = models.DateTimeField()
+
+
 class PolymorphicTests(TestCase):
     """
     The test suite
@@ -540,6 +559,76 @@ class PolymorphicTests(TestCase):
         self.assertEqual(repr(objects[2]), '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
         self.assertEqual(repr(objects[3]), '<Model2D: id 4, field1 (CharField), field2 (CharField), field3 (CharField), field4 (CharField)>')
 
+    def test_defer_fields(self):
+        self.create_model2abcd()
+
+        objects_deferred = Model2A.objects.defer('field1')
+        self.assertNotIn('field1', objects_deferred[0].__dict__,
+            'field1 was not deferred (using defer())')
+        self.assertEqual(repr(objects_deferred[0]),
+            '<Model2A_Deferred_field1: id 1, field1 (CharField)>')
+        self.assertEqual(repr(objects_deferred[1]),
+            '<Model2B_Deferred_field1: id 2, field1 (CharField), field2 (CharField)>')
+        self.assertEqual(repr(objects_deferred[2]),
+            '<Model2C_Deferred_field1: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
+        self.assertEqual(repr(objects_deferred[3]),
+            '<Model2D_Deferred_field1: id 4, field1 (CharField), field2 (CharField), field3 (CharField), field4 (CharField)>')
+
+        objects_only = Model2A.objects.only('pk', 'polymorphic_ctype', 'field1')
+        self.assertIn('field1', objects_only[0].__dict__,
+            'qs.only("field1") was used, but field1 was incorrectly deferred')
+        self.assertIn('field1', objects_only[3].__dict__,
+            'qs.only("field1") was used, but field1 was incorrectly deferred'
+            ' on a child model')
+        self.assertNotIn('field4', objects_only[3].__dict__,
+            'field4 was not deferred (using only())')
+        self.assertEqual(repr(objects_only[0]),
+            '<Model2A: id 1, field1 (CharField)>')
+        self.assertEqual(repr(objects_only[1]),
+            '<Model2B_Deferred_field2: '
+            'id 2, field1 (CharField), field2 (CharField)>')
+        self.assertEqual(repr(objects_only[2]),
+            '<Model2C_Deferred_field2_field3_model2a_ptr_id: '
+            'id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
+        self.assertEqual(repr(objects_only[3]),
+            '<Model2D_Deferred_field2_field3_field4_model2a_ptr_id_model2b_ptr_id: '
+            'id 4, field1 (CharField), field2 (CharField), field3 (CharField), field4 (CharField)>')
+
+    # A bug in Django 1.4 prevents using defer across reverse relations
+    # <https://code.djangoproject.com/ticket/14694>. Since polymorphic
+    # uses reverse relations to traverse down model inheritance, deferring
+    # fields in child models will not work in Django 1.4.
+    @skipIf(django.VERSION < (1, 5), "Django 1.4 does not support defer on related fields")
+    def test_defer_related_fields(self):
+        self.create_model2abcd()
+
+        objects_deferred_field4 = Model2A.objects.defer('Model2D___field4')
+        self.assertNotIn('field4', objects_deferred_field4[3].__dict__,
+            'field4 was not deferred (using defer(), traversing inheritance)')
+        self.assertEqual(repr(objects_deferred_field4[0]),
+            '<Model2A: id 1, field1 (CharField)>')
+        self.assertEqual(repr(objects_deferred_field4[1]),
+            '<Model2B: id 2, field1 (CharField), field2 (CharField)>')
+        self.assertEqual(repr(objects_deferred_field4[2]),
+            '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
+        self.assertEqual(repr(objects_deferred_field4[3]),
+            '<Model2D_Deferred_field4: id 4, field1 (CharField), field2 (CharField), field3 (CharField), field4 (CharField)>')
+
+        objects_only_field4 = Model2A.objects.only(
+            'polymorphic_ctype', 'field1',
+            'Model2B___id', 'Model2B___field2', 'Model2B___model2a_ptr',
+            'Model2C___id', 'Model2C___field3', 'Model2C___model2b_ptr',
+            'Model2D___id', 'Model2D___model2c_ptr')
+        self.assertEqual(repr(objects_only_field4[0]),
+            '<Model2A: id 1, field1 (CharField)>')
+        self.assertEqual(repr(objects_only_field4[1]),
+            '<Model2B: id 2, field1 (CharField), field2 (CharField)>')
+        self.assertEqual(repr(objects_only_field4[2]),
+            '<Model2C: id 3, field1 (CharField), field2 (CharField), field3 (CharField)>')
+        self.assertEqual(repr(objects_only_field4[3]),
+            '<Model2D_Deferred_field4: id 4, field1 (CharField), field2 (CharField), field3 (CharField), field4 (CharField)>')
+
+
     def test_manual_get_real_instance(self):
         self.create_model2abcd()
 
@@ -830,6 +919,31 @@ class PolymorphicTests(TestCase):
         self.assertIs(type(ModelWithMyManager._default_manager), MyManager)
         self.assertIs(type(ModelWithMyManager.base_objects), models.Manager)
 
+    def test_user_defined_manager_as_secondary(self):
+        self.create_model2abcd()
+        ModelWithMyManagerNoDefault.objects.create(field1='D1a', field4='D4a')
+        ModelWithMyManagerNoDefault.objects.create(field1='D1b', field4='D4b')
+
+        objects = ModelWithMyManagerNoDefault.my_objects.all()   # MyManager should reverse the sorting of field1
+        self.assertEqual(repr(objects[0]), '<ModelWithMyManagerNoDefault: id 6, field1 (CharField) "D1b", field4 (CharField) "D4b">')
+        self.assertEqual(repr(objects[1]), '<ModelWithMyManagerNoDefault: id 5, field1 (CharField) "D1a", field4 (CharField) "D4a">')
+        self.assertEqual(len(objects), 2)
+
+        self.assertIs(type(ModelWithMyManagerNoDefault.my_objects), MyManager)
+        self.assertIs(type(ModelWithMyManagerNoDefault.objects), PolymorphicManager)
+        self.assertIs(type(ModelWithMyManagerNoDefault._default_manager), PolymorphicManager)
+        self.assertIs(type(ModelWithMyManagerNoDefault.base_objects), models.Manager)
+
+    def test_user_objects_manager_as_secondary(self):
+        self.create_model2abcd()
+        ModelWithMyManagerDefault.objects.create(field1='D1a', field4='D4a')
+        ModelWithMyManagerDefault.objects.create(field1='D1b', field4='D4b')
+
+        self.assertIs(type(ModelWithMyManagerDefault.my_objects), MyManager)
+        self.assertIs(type(ModelWithMyManagerDefault.objects), PolymorphicManager)
+        self.assertIs(type(ModelWithMyManagerDefault._default_manager), MyManager)
+        self.assertIs(type(ModelWithMyManagerDefault.base_objects), models.Manager)
+
     @skipIf(django.VERSION < (1, 7), "This test needs Django 1.7+")
     def test_user_defined_queryset_as_manager(self):
         self.create_model2abcd()
@@ -984,6 +1098,59 @@ class PolymorphicTests(TestCase):
 
         # test that we can delete the object
         t.delete()
+        
+    def test_polymorphic__aggregate(self):
+        """ test ModelX___field syntax on aggregate (should work for annotate either) """
+        
+        Model2A.objects.create(field1='A1')
+        Model2B.objects.create(field1='A1', field2='B2')
+        Model2B.objects.create(field1='A1', field2='B2')
+        
+        # aggregate using **kwargs
+        result = Model2A.objects.aggregate(cnt=Count('Model2B___field2'))
+        self.assertEqual(result, {'cnt': 2})
+        
+        # aggregate using **args
+        with self.assertRaisesMessage(AssertionError, 'PolymorphicModel: annotate()/aggregate(): ___ model lookup supported for keyword arguments only'):
+            Model2A.objects.aggregate(Count('Model2B___field2'))
+        
+        
+        
+    @skipIf(django.VERSION < (1,8,), "This test needs Django >=1.8")
+    def test_polymorphic__complex_aggregate(self):
+        """ test (complex expression on) aggregate (should work for annotate either) """
+        
+        Model2A.objects.create(field1='A1')
+        Model2B.objects.create(field1='A1', field2='B2')
+        Model2B.objects.create(field1='A1', field2='B2')
+        
+        # aggregate using **kwargs
+        result = Model2A.objects.aggregate(
+            cnt_a1=Count(Case(When(field1='A1', then=1))),
+            cnt_b2=Count(Case(When(Model2B___field2='B2', then=1))),
+        )
+        self.assertEqual(result, {'cnt_b2': 2, 'cnt_a1': 3})
+
+        # aggregate using **args
+        # we have to set the defaul alias or django won't except a complex expression
+        # on aggregate/annotate
+        def ComplexAgg(expression):
+            complexagg = Count(expression)*10
+            complexagg.default_alias = 'complexagg'
+            return complexagg
+        
+        with self.assertRaisesMessage(AssertionError, 'PolymorphicModel: annotate()/aggregate(): ___ model lookup supported for keyword arguments only'):
+            Model2A.objects.aggregate(ComplexAgg('Model2B___field2'))
+
+    @skipIf(django.VERSION < (1,8,), "This test needs Django >=1.8")
+    def test_polymorphic__expressions(self):
+
+        from django.db.models.expressions import DateTime
+        from django.utils.timezone import utc
+
+        # no exception raised
+        result = DateModel.objects.annotate(val=DateTime('date', 'day', utc))
+        self.assertEqual(list(result), [])
 
 
 class RegressionTests(TestCase):
